__version__ = "2.1.7"
__description__ = (
    f"grabbing buried points tools. Created: Lijiawei. Version {__version__}"
)
"""
This is a python toolkit for real-time capture,
analysis, cleaning and report generation of embedded points based on the development of mitmproxy.

"""
